from enum import Enum


class Platform(str, Enum):
    win = 'win'
    linux = 'linux'
    mac = 'mac'
    win32 = 'win32'
    win64 = 'win64'
    linux32 = 'linux32'
    linux64 = 'linux64'
    mac32 = 'mac32'
    mac64 = 'mac64'
