url_chromium = 'https://chromedriver.chromium.org'
url_chromedriver_storage = 'https://chromedriver.storage.googleapis.com'
css_selector_versions = 'ul.n8H08c:nth-child(5)'
latest_stable_version_str = 'Latest stable release'
latest_beta_version_str = 'Latest beta release'
